package com.jkcredit.location.filter;

import com.jkcredit.location.service.TokenService;
import lombok.extern.slf4j.Slf4j;
import org.apache.catalina.connector.RequestFacade;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import javax.servlet.*;
import javax.servlet.annotation.WebFilter;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.UUID;

/**
 * @author xusonglin
 * @version V1.0
 **/
@WebFilter(urlPatterns = "/*", filterName = "requestCheckFilter")
@Order(value = 1)
@Component
@Slf4j
public class RequestCheckFilter implements Filter {
    @Autowired
    TokenService tokenService;

    @Override
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain) throws IOException, ServletException {
        String transationId = UUID.randomUUID().toString();
        servletRequest.setAttribute("transationId", transationId);
        if (((RequestFacade) servletRequest).getRequestURI().equals("/mrcp/gateway-ct/getToken.do")) {
            filterChain.doFilter(servletRequest, servletResponse);
        } else {
            HttpServletRequest httpServletRequest = (HttpServletRequest) servletRequest;
            String xToken = httpServletRequest.getHeader("token");
            if (StringUtils.isBlank(xToken)) {
                // token不存在
                servletRequest.getRequestDispatcher("/tokenNotExist").forward(servletRequest, servletResponse);
            } else {
                // 验证token
                String username = tokenService.validateToken(xToken);
                if (StringUtils.isBlank(username)) {
                    servletRequest.getRequestDispatcher("/tokenError").forward(servletRequest, servletResponse);
                } else {
                    servletRequest.setAttribute("username", username);
                    filterChain.doFilter(servletRequest, servletResponse);
                }
            }
        }
    }
}
