package com.jkcredit.location.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.jkcredit.location.constant.CommonConstant;
import com.jkcredit.location.enums.UserInfoEnum;
import com.jkcredit.location.model.UserObject;
import com.jkcredit.location.service.TokenService;
import com.jkcredit.location.util.JwtTokenUtil;
import io.jsonwebtoken.Claims;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.jose4j.json.JsonUtil;
import org.jose4j.jwk.RsaJsonWebKey;
import org.jose4j.lang.JoseException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import java.io.*;
import java.security.PublicKey;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author xusonglin
 * @version V1.0
 **/
@Service
@Slf4j
public class TokenServiceImpl implements TokenService {

    @Value("${userInfoPath}")
    private String userInfoPath;
    @Override
    public String getToken(String appId, String appSecret) {
        Map<String, String> claims = new HashMap<>();
        claims.put("appId", appId);
        claims.put("appSecret", appSecret);
        UserObject user = getUser(appId, appSecret);
        String token = "";
        if (user != null) {
            token = JwtTokenUtil.generateToken(user.getUsername(), claims);
        }
        return token;
    }

    @Override
    public String validateToken(String token) {
        PublicKey publicKey;
        try {
            publicKey = new RsaJsonWebKey(JsonUtil.parseJson(CommonConstant.TOKEN_PUBLIC_KEY)).getPublicKey();
        } catch (JoseException e) {
            log.error("tokenParseException:", e);
            return "";
        }
        Claims claims = JwtTokenUtil.parseToken(token, publicKey);
        if (claims == null) {
            return "";
        }
        String appId = claims.get("appId").toString();
        String appSecret = claims.get("appSecret").toString();
        String username = claims.getSubject();
        String userInfo = UserInfoEnum.getInstance().get(appId);
        if (userInfo != null) {
            String[] userInfoArray = userInfo.split("&");
            if (appSecret.equals(userInfoArray[1]) && username.equals(userInfoArray[2])) {
                return username;
            } else {
                return "";
            }
        } else {
            return "";
        }
    }

    private UserObject getUser(String appId, String appSecret) {
        UserObject userObject = new UserObject();
        ConcurrentHashMap<String, String> users = UserInfoEnum.getInstance();
        String user = users.get(appId);

        if (StringUtils.isNotBlank(user)) {
            String[] userInfo = user.split("&");
            // {"appId":"26da5c7e0e3f4a0087578aed2477cf74","appSecret":"666fc","username":"guojiao"}
            if (appSecret.equals(userInfo[1])) {
                userObject.setAppId(userInfo[0]);
                userObject.setAppSecret(userInfo[1]);
                userObject.setUsername(userInfo[2]);
                return userObject;
            } else {
                return null;
            }

        } else {
            String userJson = getUserFromJsonFile();
            if (StringUtils.isNotBlank(userJson)) {
                JSONArray userArray = JSON.parseArray(userJson);
                for (int i = 0; i < userArray.size(); i++) {
                    JSONObject jsonObject = JSON.parseObject(JSON.toJSONString(userArray.get(i)));
                    if (appId.equals(jsonObject.getString("appId"))) {
                        String userAppId = jsonObject.getString("appId");
                        String userAppSecret = jsonObject.getString("appSecret");
                        String username = jsonObject.getString("username");

                        StringBuffer stringBuffer = new StringBuffer();
                        stringBuffer.append(userAppId).append("&");
                        stringBuffer.append(userAppSecret).append("&");
                        stringBuffer.append(username);
                        UserInfoEnum.add(appId, stringBuffer.toString());

                        userObject.setAppId(userAppId);
                        userObject.setAppSecret(userAppSecret);
                        userObject.setUsername(username);
                        return userObject;
                    }
                }
            } else {
                return null;
            }
            return null;
        }
    }

    private String getUserFromJsonFile() {
        String jsonStr = "";
        File jsonFile = new File(userInfoPath);
        try {
            FileReader fileReader = new FileReader(jsonFile);
            Reader reader = new InputStreamReader(new FileInputStream(jsonFile),"utf-8");
            int ch = 0;
            StringBuffer sb = new StringBuffer();
            while ((ch = reader.read()) != -1) {
                sb.append((char) ch);
            }
            fileReader.close();
            reader.close();
            jsonStr = sb.toString();
            return jsonStr;

        } catch (IOException ioe) {
            log.error("getUserFromJsonFile.IOException:", ioe);
            return jsonStr;
        }
    }
}
