package com.jkcredit.location.util;

import com.jkcredit.location.constant.CommonConstant;
import io.jsonwebtoken.Claims;
import io.jsonwebtoken.Jwts;
import lombok.extern.slf4j.Slf4j;
import org.jose4j.json.JsonUtil;
import org.jose4j.jwk.RsaJsonWebKey;
import org.jose4j.jws.AlgorithmIdentifiers;
import org.jose4j.jws.JsonWebSignature;
import org.jose4j.jwt.JwtClaims;
import org.jose4j.jwt.NumericDate;
import org.jose4j.lang.JoseException;

import java.security.PrivateKey;
import java.security.PublicKey;
import java.util.Map;

@Slf4j
public class JwtTokenUtil {
    public static String generateToken(String subject, Map<String, String> user) {
        JwtClaims claims = new JwtClaims();
        claims.setGeneratedJwtId();
        claims.setIssuedAtToNow();
        //过期时间一定要设置，并且小于7天
        NumericDate date = NumericDate.now();
        date.addSeconds(30*60);
        claims.setExpirationTime(date);
        claims.setSubject(subject);
        //添加自定义参数，所有值请都使用String类型
        claims.setClaim("appId", user.get("appId"));
        claims.setClaim("appSecret", user.get("appSecret"));

        JsonWebSignature jws = new JsonWebSignature();
        jws.setAlgorithmHeaderValue(AlgorithmIdentifiers.RSA_USING_SHA256);
        //必须设置
        jws.setKeyIdHeaderValue(CommonConstant.KEY_ID);
        jws.setPayload(claims.toJson());
        String jwtResult = "";
        try {
            PrivateKey privateKey = new RsaJsonWebKey(JsonUtil.parseJson(CommonConstant.TOKEN_PRIVATE_KEY)).getPrivateKey();
            jws.setKey(privateKey);
            jwtResult = jws.getCompactSerialization();
        } catch (JoseException e) {
            e.printStackTrace();
            log.error("generateToken.JoseException:", e);
        }
        return jwtResult;
    }

    public static Claims parseToken(String token, PublicKey salt) {
        try {
            Claims claims = Jwts.parser()
                    .setSigningKey(salt)
                    .parseClaimsJws(token).getBody();
            return claims;
        } catch (Exception e) {
            log.error("parseToken.Exception:", e);
        }
        return null;
    }
}
